==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ronportal/dochazka/smenaproden.php <==
<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\PracovniPomer;

class SmenaProDen
{
    var $datum;

    var $kod;
    var $popis;
    var $variabilni;
    // Casy jsou v minutach (pro prevod na timestamp vynasobit *60, pro zobrazeni pouzit DateUtils::minutesToString)
    var $pocatekSmeny;
    var $konecSmeny;
    var $delkaSmeny;

    var $pocatekPevneCasti;
    var $konecPevneCasti;
    var $delkaPevneCasti;

    var $delena;
    var $preruseniOd;
    var $preruseniDo;

    /**
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @return SmenaProDen[]
     */
    static function nactiSmenySQL($database, $sql, $params)
    {
        $result = [];

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $smena = new SmenaProDen();

            $smena ->datum = $database ->stringToTimestamp($row[0]);

            $smena ->kod = $row[1];
            $smena ->popis = $row[2];
            $smena ->variabilni = (int)$row[3];

            if ($smena ->variabilni) {
                $smena ->pocatekSmeny = $database ->stringToTimeOfDay($row[10]) * 60;
                $smena ->konecSmeny = $database ->stringToTimeOfDay($row[11]) * 60;
                $smena ->delkaSmeny = $database ->stringToTimeOfDay($row[12]) * 60;
            } else {
                $smena ->pocatekSmeny = $database ->stringToTimeOfDay($row[4]) * 60;
                $smena ->konecSmeny = $database ->stringToTimeOfDay($row[5]) * 60;
                $smena ->delkaSmeny = $database ->stringToTimeOfDay($row[6]) * 60;
            }

            $smena ->pocatekPevneCasti = $row[7] ? $database ->stringToTimeOfDay($row[7]) * 60 : null;
            $smena ->konecPevneCasti = $row[8] ? $database ->stringToTimeOfDay($row[8]) * 60 : null;
            $smena ->delkaPevneCasti = $row[9] ? $database ->stringToTimeOfDay($row[9]) * 60 : null;

            $smena ->delena = (int)$row[13];
            if ($smena ->delena) {
                $smena ->preruseniOd = $row[14] ? $database ->stringToTimeOfDay($row[14]) * 60 : null;
                $smena ->preruseniDo = $row[15] ? $database ->stringToTimeOfDay($row[15]) * 60 : null;
            }

            $result[] = $smena;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param PracovniPomer|string $pp_rc
     * @param int $datumOd
     * @param int $datumDo
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleKapacit($database, $pp_rc, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PlanovaniKapacit.Datum, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka, ' . // 10..12
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo ' . // 13..15
            '  FROM PlanovaniKapacit, Smena' .
            ' WHERE PlanovaniKapacit.Smena = Smena.Kod ' .
            '   AND PlanovaniKapacit.Datum >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND PlanovaniKapacit.Datum <= ' . $params ->addParameter($database ->dateToParam($datumDo));

        if ($pp_rc instanceof PracovniPomer) {
            $sql .=
                '   AND PlanovaniKapacit.RC = ' . $params->addParameter($pp_rc->rc) .
                '   AND PlanovaniKapacit.OsobniCislo = ' . $params->addParameter($pp_rc->osobniCislo) .
                '   AND PlanovaniKapacit.DatumNastupu = ' . $params->addParameter($database->dateToParam($pp_rc->datumNastupu));
        } else {
            $sql .=
                '   AND PlanovaniKapacit.RC = ' . $params->addParameter($pp_rc);
        }

        $sql .=
            ' ORDER BY PlanovaniKapacit.Datum ';

        MessageManager::addDebugLog('nactiSmenyPodleKapacit: ' . $sql);
        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param string $filter
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleKalendare($database, $pp, $datumOd, $datumDo, $filter = '')
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT SmenaProDenKalendare.Den, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo ' . // 13..15
            '  FROM PrirazeniKalendare, SmenaProDenKalendare, Smena ' .
            ' WHERE SmenaProDenKalendare.Kalendar = PrirazeniKalendare.Kalendar ' .
            '   AND PrirazeniKalendare.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PrirazeniKalendare.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND PrirazeniKalendare.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)).
            '   AND PrirazeniKalendare.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND (PrirazeniKalendare.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR PrirazeniKalendare.PlatnostDo IS NULL) ' .
            '   AND SmenaProDenKalendare.Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND SmenaProDenKalendare.Den <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND PrirazeniKalendare.PlatnostOd <= SmenaProDenKalendare.Den' .
            '   AND (PrirazeniKalendare.PlatnostDo >= SmenaProDenKalendare.Den OR PrirazeniKalendare.PlatnostDo IS NULL)' .
            '   AND Smena.Kod = SmenaProDenKalendare.Smena' .
            ($filter ? ' AND (' . $filter . ')' : '') .
            ' ORDER BY SmenaProDenKalendare.Den ';

        MessageManager::addDebugLog('nactiSmenyPodleKalendare: ' . $sql);
        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * Nacte smeny podle kalendare prirazeneho k rozdeleni osoby
     * @param Database $database
     * @param int $kodTypuRozdeleni
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleKalendareRozdeleni($database, $kodTypuRozdeleni, $pp, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT SmenaProDenKalendare.Den, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo ' . // 13..15
            '  FROM RozdeleniOsob, Rozdeleni, SmenaProDenKalendare, Smena ' .
            ' WHERE SmenaProDenKalendare.Kalendar = Rozdeleni.Kalendar ' .
            '   AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
            '   AND Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$kodTypuRozdeleni) .
            '   AND RozdeleniOsob.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND RozdeleniOsob.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)).
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND SmenaProDenKalendare.Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND SmenaProDenKalendare.Den <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND RozdeleniOsob.PlatnostOd <= SmenaProDenKalendare.Den' .
            '   AND (RozdeleniOsob.PlatnostDo >= SmenaProDenKalendare.Den OR RozdeleniOsob.PlatnostDo IS NULL)' .
            '   AND Smena.Kod = SmenaProDenKalendare.Smena' .
            ' ORDER BY SmenaProDenKalendare.Den ';

        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * Nacte smeny podle kalendare prirazeneho ke zmenenemu rozdeleni osoby (prerazeni v planovani kapacit).
     * POZOR! Muze vracet i zaznamy, ktere maji vse krome datumu NULL (zamestnanec muze byt prerazen na jine
     * rozdeleni, ale jeste nemusi mit prirazenu smenu)
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $puvodniPlan
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleZmenenehoRozdeleni($database, $pp, $datumOd, $datumDo, $puvodniPlan)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PlanovaniKapacit.Datum, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo ' . // 13..15
            ' FROM PlanovaniKapacit INNER JOIN Rozdeleni ' .
            ($puvodniPlan
                ? ' ON Rozdeleni.KodTypuRozdeleni = PlanovaniKapacit.PuvodniPlan_KodTypu' .
                '   AND Rozdeleni.Kod = PlanovaniKapacit.PuvodniPlan_Rozdeleni'
                : ' ON Rozdeleni.KodTypuRozdeleni = PlanovaniKapacit.KodTypuRozdeleni' .
                '   AND Rozdeleni.Kod = PlanovaniKapacit.Rozdeleni'
            ) .
            ' LEFT OUTER JOIN SmenaProDenKalendare ' .
            '  ON SmenaProDenKalendare.Kalendar = Rozdeleni.Kalendar ' .
            '  AND SmenaProDenKalendare.Den = PlanovaniKapacit.Datum' .
            ' LEFT OUTER JOIN Smena ' .
            '  ON Smena.Kod = SmenaProDenKalendare.Smena' .
            ' WHERE PlanovaniKapacit.Datum <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND PlanovaniKapacit.Datum >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND PlanovaniKapacit.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PlanovaniKapacit.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND PlanovaniKapacit.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)).
            ' ORDER BY PlanovaniKapacit.Datum ';

        // POZOR! Vse krome datumu muze byt NULL!
        return self::nactiSmenySQL($database, $sql, $params);
    }


    public function format($format = null)
    {
        if (!isset($format)) {
            return
                '<span title="' . StringUtils::htmlTextConvert($this ->popis) . '">' . StringUtils::htmlTextConvert($this ->kod) . '</span>';
        }

        $result = str_replace('%POPIS%', $this ->popis, $format);
        $result = str_replace('%KOD%', $this ->kod, $result);
        $result = str_replace('%DELKA%', StringUtils::format('minuty', $this ->delkaSmeny), $result);

        return $result;
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2206_162.zip: ronportal/ronportal/dochazka/smenaproden.php <==
<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\PracovniPomer;



class SmenaProDen
{
    public $datum;

    public $kod;
    public $popis;
    public $variabilni;
    // Casy jsou v minutach (pro prevod na timestamp vynasobit *60, pro zobrazeni pouzit DateUtils::minutesToString)
    public $pocatekSmeny;
    public $konecSmeny;
    public $delkaSmeny;

    public $pocatekPevneCasti;
    public $konecPevneCasti;
    public $delkaPevneCasti;

    public $delena;
    public $preruseniOd;
    public $preruseniDo;

    /**
     * Nacte seznam prirazenych smen ze zadaneho SQL dotazu
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @return SmenaProDen[]
     */
    private static function nactiSmenySQL($database, $sql, $params)
    {
        $result = [];

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $smena = new SmenaProDen();

            $smena ->datum = $database ->stringToTimestamp($row[0]);

            $smena ->kod = $row[1];
            $smena ->popis = TextManager::_('db_smena_popis_' . $row[1], $row[2]);
            $smena ->variabilni = (int)$row[3];

            if ($smena ->variabilni) {
                $smena ->pocatekSmeny = $database ->stringToTimeOfDay($row[10]) * 60;
                $smena ->konecSmeny = $database ->stringToTimeOfDay($row[11]) * 60;
                $smena ->delkaSmeny = $database ->stringToTimeOfDay($row[12]) * 60;
            } else {
                $smena ->pocatekSmeny = $database ->stringToTimeOfDay($row[4]) * 60;
                $smena ->konecSmeny = $database ->stringToTimeOfDay($row[5]) * 60;
                $smena ->delkaSmeny = $database ->stringToTimeOfDay($row[6]) * 60;
            }

            $smena ->pocatekPevneCasti = $row[7] ? $database ->stringToTimeOfDay($row[7]) * 60 : null;
            $smena ->konecPevneCasti = $row[8] ? $database ->stringToTimeOfDay($row[8]) * 60 : null;
            $smena ->delkaPevneCasti = $row[9] ? $database ->stringToTimeOfDay($row[9]) * 60 : null;

            $smena ->delena = (int)$row[13];
            if ($smena ->delena) {
                $smena ->preruseniOd = $row[14] ? $database ->stringToTimeOfDay($row[14]) * 60 : null;
                $smena ->preruseniDo = $row[15] ? $database ->stringToTimeOfDay($row[15]) * 60 : null;
            }

            $result[] = $smena;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * Nacte smeny zadane osoby z planovani kapacit
     * @param Database $database
     * @param PracovniPomer|string $pp_rc
     * @param int $datumOd
     * @param int $datumDo
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleKapacit($database, $pp_rc, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PlanovaniKapacit.Datum, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka, ' . // 10..12
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo ' . // 13..15
            '  FROM PlanovaniKapacit, Smena' .
            ' WHERE PlanovaniKapacit.Smena = Smena.Kod ' .
            '   AND PlanovaniKapacit.Datum >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND PlanovaniKapacit.Datum <= ' . $params ->addParameter($database ->dateToParam($datumDo));

        if ($pp_rc instanceof PracovniPomer) {
            $sql .=
                '   AND PlanovaniKapacit.RC = ' . $params->addParameter($pp_rc->rc) .
                '   AND PlanovaniKapacit.OsobniCislo = ' . $params->addParameter($pp_rc->osobniCislo) .
                '   AND PlanovaniKapacit.DatumNastupu = ' . $params->addParameter($database->dateToParam($pp_rc->datumNastupu));
        } else {
            $sql .=
                '   AND PlanovaniKapacit.RC = ' . $params->addParameter($pp_rc);
        }

        $sql .=
            ' ORDER BY PlanovaniKapacit.Datum ';

        MessageManager::addDebugLog('nactiSmenyPodleKapacit: ' . $sql);
        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * Nacte smeny osoby z prirazenych kalendaru
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param string $filter
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleKalendare($database, $pp, $datumOd, $datumDo, $filter = '')
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT SmenaProDenKalendare.Den, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo ' . // 13..15
            '  FROM PrirazeniKalendare, SmenaProDenKalendare, Smena ' .
            ' WHERE SmenaProDenKalendare.Kalendar = PrirazeniKalendare.Kalendar ' .
            '   AND PrirazeniKalendare.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PrirazeniKalendare.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND PrirazeniKalendare.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)).
            '   AND PrirazeniKalendare.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND (PrirazeniKalendare.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR PrirazeniKalendare.PlatnostDo IS NULL) ' .
            '   AND SmenaProDenKalendare.Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND SmenaProDenKalendare.Den <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND PrirazeniKalendare.PlatnostOd <= SmenaProDenKalendare.Den' .
            '   AND (PrirazeniKalendare.PlatnostDo >= SmenaProDenKalendare.Den OR PrirazeniKalendare.PlatnostDo IS NULL)' .
            '   AND Smena.Kod = SmenaProDenKalendare.Smena' .
            ($filter ? ' AND (' . $filter . ')' : '') .
            ' ORDER BY SmenaProDenKalendare.Den ';

        MessageManager::addDebugLog('nactiSmenyPodleKalendare: ' . $sql);
        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @return SmenaProDen[]
     */
    static function nactiSmenyOsoby($database, $pp, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $sql =
            // smeny pro kalendare, ktere se nepocitaji podle planovani kapacit
            'SELECT SmenaProDenKalendare.Den, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo ' . // 13..15
            "\nFROM PrirazeniKalendare, SmenaProDenKalendare, Smena, Kalendar\n" .
            ' WHERE SmenaProDenKalendare.Kalendar = PrirazeniKalendare.Kalendar ' .
            '   AND PrirazeniKalendare.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PrirazeniKalendare.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND PrirazeniKalendare.DatumNastupu = ' . $params ->addDateParameter($pp->datumNastupu) .
            '   AND SmenaProDenKalendare.Den >= ' . $params ->addDateParameter($datumOd) .
            '   AND SmenaProDenKalendare.Den <= ' . $params ->addDateParameter($datumDo) .
            '   AND PrirazeniKalendare.PlatnostOd <= SmenaProDenKalendare.Den' .
            '   AND (PrirazeniKalendare.PlatnostDo >= SmenaProDenKalendare.Den OR PrirazeniKalendare.PlatnostDo IS NULL)' .
            '   AND Smena.Kod = SmenaProDenKalendare.Smena' .
            '   AND Kalendar.Kod = PrirazeniKalendare.Kalendar' .
            '   AND Kalendar.DochazkaPodlePlanKapacit = 0' .

            "\nUNION\n" .

            'SELECT PlanovaniKapacit.Datum, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka, ' . // 10..12
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo ' . // 13..15
            "\nFROM PrirazeniKalendare, PlanovaniKapacit, Smena, Kalendar\n" .
            ' WHERE PlanovaniKapacit.Smena = Smena.Kod ' .
            '   AND PlanovaniKapacit.Datum >= ' . $params ->addDateParameter($datumOd) .
            '   AND PlanovaniKapacit.Datum <= ' . $params ->addDateParameter($datumDo) .
            '   AND PlanovaniKapacit.RC = ' . $params->addParameter($pp->rc) .
            '   AND PlanovaniKapacit.OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
            '   AND PlanovaniKapacit.DatumNastupu = ' . $params->addDateParameter($pp->datumNastupu) .
            '   AND PrirazeniKalendare.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PrirazeniKalendare.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND PrirazeniKalendare.DatumNastupu = ' . $params ->addDateParameter($pp->datumNastupu) .
            '   AND PrirazeniKalendare.PlatnostOd <= PlanovaniKapacit.Datum' .
            '   AND (PrirazeniKalendare.PlatnostDo >= PlanovaniKapacit.Datum OR PrirazeniKalendare.PlatnostDo IS NULL)' .
            '   AND Kalendar.Kod = PrirazeniKalendare.Kalendar' .
            '   AND Kalendar.DochazkaPodlePlanKapacit = 1' .

            "\nORDER BY 1";

        MessageManager::addDebugLog('nactiSmenyOsoby: ' . $sql);
        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * Nacte smeny podle kalendare prirazeneho k rozdeleni zadane osoby
     * @param Database $database
     * @param int $kodTypuRozdeleni
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleKalendareRozdeleni($database, $kodTypuRozdeleni, $pp, $datumOd, $datumDo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT SmenaProDenKalendare.Den, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo ' . // 13..15
            '  FROM RozdeleniOsob, Rozdeleni, SmenaProDenKalendare, Smena ' .
            ' WHERE SmenaProDenKalendare.Kalendar = Rozdeleni.Kalendar ' .
            '   AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
            '   AND Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$kodTypuRozdeleni) .
            '   AND RozdeleniOsob.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND RozdeleniOsob.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)).
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datumOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND SmenaProDenKalendare.Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND SmenaProDenKalendare.Den <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND RozdeleniOsob.PlatnostOd <= SmenaProDenKalendare.Den' .
            '   AND (RozdeleniOsob.PlatnostDo >= SmenaProDenKalendare.Den OR RozdeleniOsob.PlatnostDo IS NULL)' .
            '   AND Smena.Kod = SmenaProDenKalendare.Smena' .
            ' ORDER BY SmenaProDenKalendare.Den ';

        return self::nactiSmenySQL($database, $sql, $params);
    }


    /**
     * Nacte smeny podle kalendare prirazeneho ke zmenenemu rozdeleni osoby (prerazeni v planovani kapacit).
     * POZOR! Muze vracet i zaznamy, ktere maji vse krome datumu NULL (zamestnanec muze byt prerazen na jine
     * rozdeleni, ale jeste nemusi mit prirazenu smenu)
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $puvodniPlan
     * @return SmenaProDen[]
     */
    static function nactiSmenyPodleZmenenehoRozdeleni($database, $pp, $datumOd, $datumDo, $puvodniPlan)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT PlanovaniKapacit.Datum, Smena.Kod, Smena.Popis, Smena.Variabilni, ' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 4..6
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevneCasti, ' . // 7..9
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 10..12 - pro variabilni smenu
            ' Smena.Delena, Smena.PreruseniOd, Smena.PreruseniDo ' . // 13..15
            ' FROM PlanovaniKapacit INNER JOIN Rozdeleni ' .
            ($puvodniPlan
                ? ' ON Rozdeleni.KodTypuRozdeleni = PlanovaniKapacit.PuvodniPlan_KodTypu' .
                '   AND Rozdeleni.Kod = PlanovaniKapacit.PuvodniPlan_Rozdeleni'
                : ' ON Rozdeleni.KodTypuRozdeleni = PlanovaniKapacit.KodTypuRozdeleni' .
                '   AND Rozdeleni.Kod = PlanovaniKapacit.Rozdeleni'
            ) .
            ' LEFT OUTER JOIN SmenaProDenKalendare ' .
            '  ON SmenaProDenKalendare.Kalendar = Rozdeleni.Kalendar ' .
            '  AND SmenaProDenKalendare.Den = PlanovaniKapacit.Datum' .
            ' LEFT OUTER JOIN Smena ' .
            '  ON Smena.Kod = SmenaProDenKalendare.Smena' .
            ' WHERE PlanovaniKapacit.Datum <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            '   AND PlanovaniKapacit.Datum >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            '   AND PlanovaniKapacit.RC = ' . $params ->addParameter($pp ->rc) .
            '   AND PlanovaniKapacit.OsobniCislo = ' . $params ->addParameter($pp->osobniCislo) .
            '   AND PlanovaniKapacit.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp->datumNastupu)).
            ' ORDER BY PlanovaniKapacit.Datum ';

        // POZOR! Vse krome datumu muze byt NULL!
        return self::nactiSmenySQL($database, $sql, $params);
    }


    public function format($format = null)
    {
        if (!isset($format)) {
            return
                '<span title="' . StringUtils::htmlTextConvert($this ->popis) . '">' . StringUtils::htmlTextConvert($this ->kod) . '</span>';
        }

        $result = str_replace('%POPIS%', $this ->popis, $format);
        $result = str_replace('%KOD%', $this ->kod, $result);
        $result = str_replace('%DELKA%', StringUtils::format('minuty', $this ->delkaSmeny), $result);

        return $result;
    }
}
